# Generated by Django 2.2 on 2019-06-11 13:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_walletpass', '0003_auto_20190610_1434'),
    ]

    operations = [
        migrations.AlterField(
            model_name='pass',
            name='authentication_token',
            field=models.CharField(max_length=150),
        ),
    ]
