# Generated by Django 2.2 on 2019-06-13 16:07

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_walletpass', '0005_auto_20190613_1433'),
    ]

    operations = [
        migrations.AlterField(
            model_name='registration',
            name='pazz',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='registrations', to='django_walletpass.Pass'),
        ),
    ]
