# Generated by Django 2.2 on 2019-06-13 14:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_walletpass', '0004_auto_20190611_1306'),
    ]

    operations = [
        migrations.AlterField(
            model_name='pass',
            name='pass_type_identifier',
            field=models.CharField(max_length=150),
        ),
        migrations.AlterField(
            model_name='pass',
            name='serial_number',
            field=models.CharField(max_length=150),
        ),
        migrations.AlterField(
            model_name='registration',
            name='device_library_identifier',
            field=models.CharField(max_length=150),
        ),
        migrations.AlterField(
            model_name='registration',
            name='push_token',
            field=models.CharField(max_length=150),
        ),
    ]
