# Generated by Django 2.2 on 2019-04-29 18:19

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_walletpass', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='pass',
            options={'verbose_name_plural': 'passes'},
        ),
    ]
